c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine calc_tau_rayleigh(pure_co2,g,lambda,p1,p2,
     &     tau_rayleigh)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the Rayleigh optical thickness between two pressure levels,
c     for pure CO2 atmospheres and terrestrial air (CO2/N2 mixtures)
c
c     This routine has been validated against values of Rayleigh optical thicknesses found in table 3 (p. 1860)
c     of "On Rayleigh optical depth calculations", B.A. Bodhaine & al., Journal of atmospheric and oceanic technology
c     volume 16, pp. 1854-1861 (1999).
c
c     Input:
c       + pure_co2: true if the gas is pure CO2; false if it is a CO2/N2 mixture or terrestrial air
c       + g: acceleration of gravity (m/s^2)
c       + lambda: wavelength (micrometers)
c       + p1: first pressure level (highest value) (Pa)
c       + p2: second pressure level (lowest value) (Pa)
c
c     Output:
c       + tau_rayleigh: Rayleigh optical thickness
c
c     I/O
      logical pure_co2
      double precision g
      double precision lambda
      double precision p1
      double precision p2
      double precision tau_rayleigh
c     temp
      double precision alpha
      double precision P0
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine calc_tau_rayleigh'

      if (p2.gt.p1) then
         call error(label)
         write(*,*) 'p2=',p2
         write(*,*) '> p1=',p1
         stop
      endif

      if (p1.lt.0.0D+0) then
         call error(label)
         write(*,*) 'p1=',p1
         stop
      endif

      if (p2.lt.0.0D+0) then
         call error(label)
         write(*,*) 'p2=',p2
         stop
      endif

      P0=9.423D+6 ! Rayleigh scattering reference pressure in pascals.

      if (pure_co2) then ! pure CO2 atmosphere
         alpha=(8.70D+0/g)
     &        *(1.527D+0*
     &        (1.0D+0+0.013D+0/lambda**2.0D+0)
     &        /lambda**4.0D+0)
     &        /P0 ! Pa^{-1}
      else ! CO2/N2 mixture
         alpha=(9.81D+0/g)
     &        *(
     &        8.569D-3*(
     &        1.0D+0+0.0113D+0/lambda**2+0.00013D+0/lambda**4.0D+0
     &        )/lambda**4.0D+0
     &        )
     &        /(P0/93.0D+0) ! Pa^{-1}
      endif

      tau_rayleigh=alpha*(p1-p2) ! unitless

      return
      end
